
local CustomDecoration = ...

local _PrefixAsset = "CUSTOM_DECORATION_V1_"

-- Apply building asset processor
CustomDecoration:registerAssetProcessor("models/decorations/decoration.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	CustomDecoration:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                }--,
--[[	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 0 }
		    }
	        }
]]
	})
end

-- Register Main prefab nodes
CustomDecoration:registerAssetId("models/decorations/decoration.fbx/Prefab/decoration/Minor/step_1_Ladder_Tiling1Part", _PrefixAsset .. "TILING1_PREFAB")
CustomDecoration:registerAssetId("models/decorations/decoration.fbx/Prefab/decoration/Minor/step_2_Ladder_Tiling2Part", _PrefixAsset .. "TILING2_PREFAB")
--------------------------------------------------------------------------

local defaultNodePrefixList = {
	_PrefixAsset .. "TILING1",
	_PrefixAsset .. "TILING2"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Create default minor decoration part registering function 
function registerDefaultMinorDecorationPart(_nameFbx, _nameAsset, _stoneCost)
    CustomDecoration:registerAssetId("models/decorations/decoration.fbx/Prefab/decoration/Minor/" .. _nameFbx .. "Part", _PrefixAsset ..  _nameAsset .. "_PREFAB")

    function registerDecorationPart(_prefixAssetTop, _moverType)
        CustomDecoration:register({
        	DataType = "BUILDING_PART",
	        Id = _prefixAssetTop .. _nameAsset .. "_PART",
	        Name = _prefixAssetTop .. _nameAsset .. "_NAME",
	        Description = _prefixAssetTop .. _nameAsset .. "_DESC",
	        Category = "DECORATION",
	        Mover = { DataType = _moverType },
	        ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		    CoreObjectPrefab = _PrefixAsset .. _nameAsset .. "_PREFAB",
                    IsPreviewGroundVisible = false
	        },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
	            },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _stoneCost }
		    }
	        }
        })
    end

    registerDecorationPart(_PrefixAsset .. "TOP_", "BUILDING_PART_MOVER")
    registerDecorationPart("STONE_GATE_V5_DECORATION_", "BUILDING_PART_MOVER_INSTANCE")

    CustomDecoration:registerPrefabComponent("models/decorations/decoration.fbx/Prefab/decoration/Minor/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the facades)
  	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
    })

-- Register Main prefab nodes
    CustomDecoration:registerAssetId("models/decorations/decoration.fbx/Prefab/decoration/Minor/step_0_Ladder_" .. _nameFbx .. "_RootPart", _PrefixAsset .. _nameAsset .. "_ROOT_PREFAB")

-- Register the root part
    CustomDecoration:register({
	DataType = "BUILDING_PART",
	Id = _PrefixAsset .. _nameAsset .. "_ROOT_PART",
	Name = _PrefixAsset .. _nameAsset .. "_ROOT_NAME",
	Description = _PrefixAsset .. _nameAsset .. "_ROOT_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = _PrefixAsset .. _nameAsset .. "_ROOT_PREFAB",
		EndPart = _PrefixAsset .. "TOP_" .. _nameAsset .. "_PART",
		FillerList = {
			_PrefixAsset .. "TILING2_PART"
                },
		BasementFillerList = {
			_PrefixAsset .. "TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 0 }
		}
	}
    })

-- Override building part components properties (generated by the building asset processor) to add building zones
    CustomDecoration:registerPrefabComponent("models/decorations/decoration.fbx/Prefab/decoration/Minor/step_0_Ladder_" .. _nameFbx .. "_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0.01, 0.01 }
    })
end

registerDefaultMinorDecorationPart("SFenetre", "SFENETRE", 1)
registerDefaultMinorDecorationPart("LFenetre", "LFENETRE", 1)
registerDefaultMinorDecorationPart("PFenetre", "PFENETRE", 1)
registerDefaultMinorDecorationPart("ArrowSlit", "ARROWSLIT", 1)
registerDefaultMinorDecorationPart("Lucarne", "LUCARNE", 1)
registerDefaultMinorDecorationPart("Balcony", "BALCONY", 2)
registerDefaultMinorDecorationPart("Chimney", "CHIMNEY", 1)
registerDefaultMinorDecorationPart("Tourelle", "TURRET", 2)

-- Register air vent particle system (smoke)
    CustomDecoration:registerAssetId("models/decorations/decoration.fbx/Prefab/decoration/Minor/ChimneyPart/SmokeEmitter", "CUSTOM_CHIMNEY_SMOKE_EMITTER_PREFAB")

    CustomDecoration:registerPrefabComponent("models/decorations/decoration.fbx/Prefab/decoration/Minor/ChimneyPart", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
                 { "CUSTOM_CHIMNEY_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
    })

    CustomDecoration:registerPrefabComponent("models/decorations/decoration.fbx/Prefab/decoration/Minor/ChimneyPart/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
	IsPlaying = true,
	IsEmitting = true
    })

    CustomDecoration:registerPrefabComponent("models/decorations/decoration.fbx/Prefab/decoration/Minor/ChimneyPart/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })

-- Create specific decoration part registering function 
function registerDefaultMajorDecorationPart(_nameFbx, _nameAsset, _stoneCost)
    CustomDecoration:registerAssetId("models/decorations/decoration.fbx/Prefab/decoration/Minor/" .. _nameFbx .. "Part", _PrefixAsset ..  _nameAsset .. "_PREFAB")

    function registerDecorationPart(_prefixAssetTop, _moverType)
        CustomDecoration:register({
        	DataType = "BUILDING_PART",
	        Id = _prefixAssetTop .. _nameAsset .. "_PART",
	        Name = _prefixAssetTop .. _nameAsset .. "_NAME",
	        Description = _prefixAssetTop .. _nameAsset .. "_DESC",
	        Category = "DECORATION",
	        Mover = { DataType = _moverType },
	        ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		    CoreObjectPrefab = _PrefixAsset .. _nameAsset .. "_PREFAB",
                    IsPreviewGroundVisible = false
	        },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
	            },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _stoneCost }
		    }
	        }
        })
    end

    registerDecorationPart(_PrefixAsset .. "TOP_", "BUILDING_PART_MOVER")
--    registerDecorationPart("STONE_GATE_V5_DECORATION_", "BUILDING_PART_MOVER_INSTANCE")

    CustomDecoration:registerPrefabComponent("models/decorations/decoration.fbx/Prefab/decoration/Minor/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the facades)
  	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
    })

-- Register Main prefab nodes
    CustomDecoration:registerAssetId("models/decorations/decoration.fbx/Prefab/decoration/Minor/step_0_Ladder_" .. _nameFbx .. "_RootPart", _PrefixAsset .. _nameAsset .. "_ROOT_PREFAB")

-- Register the root part
    CustomDecoration:register({
	DataType = "BUILDING_PART",
	Id = _PrefixAsset .. _nameAsset .. "_ROOT_PART",
	Name = _PrefixAsset .. _nameAsset .. "_ROOT_NAME",
	Description = _PrefixAsset .. _nameAsset .. "_ROOT_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = _PrefixAsset .. _nameAsset .. "_ROOT_PREFAB",
		EndPart = _PrefixAsset .. "TOP_" .. _nameAsset .. "_PART",
		FillerList = {
			_PrefixAsset .. "TILING2_PART"
                },
		BasementFillerList = {
			_PrefixAsset .. "TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 0 }
		}
	}
    })

-- Override building part components properties (generated by the building asset processor) to add building zones
    CustomDecoration:registerPrefabComponent("models/decorations/decoration.fbx/Prefab/decoration/Minor/step_0_Ladder_" .. _nameFbx .. "_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0.01, 0.01 }
    })
end

registerDefaultMajorDecorationPart("DoorA", "DOOR_A", 1)
registerDefaultMajorDecorationPart("DoorB", "DOOR_B", 1)

-- Create wooden decoration part registering function 
CustomDecoration:registerAssetProcessor("models/decorations/woodenDecoration.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
CustomDecoration:registerAssetProcessor("models/decorations/woodenDecoration2.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerDefaultWoodenDecorationPart(_nameFbxBase, _nameFbx, _nameAsset, _stoneCost)
    CustomDecoration:registerAssetId("models/decorations/" .. _nameFbxBase .. ".fbx/Prefab/woodenDecoration/" .. _nameFbx .. "Part", _PrefixAsset ..  _nameAsset .. "_PREFAB")

    function registerDecorationPart(_prefixAssetTop, _moverType)
        CustomDecoration:register({
        	DataType = "BUILDING_PART",
	        Id = _prefixAssetTop .. _nameAsset .. "_PART",
	        Name = _prefixAssetTop .. _nameAsset .. "_NAME",
	        Description = _prefixAssetTop .. _nameAsset .. "_DESC",
	        Category = "DECORATION",
	        Mover = { DataType = _moverType },
	        ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		    CoreObjectPrefab = _PrefixAsset .. _nameAsset .. "_PREFAB",
                    IsPreviewGroundVisible = false
	        },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
	            },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _stoneCost }
		    }
	        }
        })
    end

    registerDecorationPart(_PrefixAsset .. "TOP_", "BUILDING_PART_MOVER")
--    registerDecorationPart("STONE_GATE_V5_DECORATION_", "BUILDING_PART_MOVER_INSTANCE")

    CustomDecoration:registerPrefabComponent("models/decorations/" .. _nameFbxBase .. ".fbx/Prefab/woodenDecoration/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the facades)
  	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
    })

-- Register Main prefab nodes
    CustomDecoration:registerAssetId("models/decorations/" .. _nameFbxBase .. ".fbx/Prefab/woodenDecoration/step_0_Ladder_" .. _nameFbx .. "_RootPart", _PrefixAsset .. _nameAsset .. "_ROOT_PREFAB")

-- Register the root part
    CustomDecoration:register({
	DataType = "BUILDING_PART",
	Id = _PrefixAsset .. _nameAsset .. "_ROOT_PART",
	Name = _PrefixAsset .. _nameAsset .. "_ROOT_NAME",
	Description = _PrefixAsset .. _nameAsset .. "_ROOT_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = _PrefixAsset .. _nameAsset .. "_ROOT_PREFAB",
		EndPart = _PrefixAsset .. "TOP_" .. _nameAsset .. "_PART",
		FillerList = {
			_PrefixAsset .. "TILING2_PART"
                },
		BasementFillerList = {
			_PrefixAsset .. "TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "WOOD", Quantity = 0 }
		}
	}
    })

-- Override building part components properties (generated by the building asset processor) to add building zones
    CustomDecoration:registerPrefabComponent("models/decorations/" .. _nameFbxBase .. ".fbx/Prefab/woodenDecoration/step_0_Ladder_" .. _nameFbx .. "_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0.01, 0.01 }
    })
end

registerDefaultWoodenDecorationPart("woodenDecoration", "door1", "WOODEN_DOOR_1", 1)
registerDefaultWoodenDecorationPart("woodenDecoration", "door2", "WOODEN_DOOR_2", 1)
registerDefaultWoodenDecorationPart("woodenDecoration", "door3", "WOODEN_DOOR_3", 1)
registerDefaultWoodenDecorationPart("woodenDecoration", "window1", "WOODEN_WINDOW_1", 1)
registerDefaultWoodenDecorationPart("woodenDecoration2", "window2", "WOODEN_WINDOW_2", 1)
registerDefaultWoodenDecorationPart("woodenDecoration2", "window3", "WOODEN_WINDOW_3", 1)

